﻿using System.Collections.Generic;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace SampleApp_MS3_7583
{
    /// <summary>
    /// Interaction logic for TestWindow.xaml
    /// </summary>
    public partial class TestWindow : Window
    {
        private ManagedProj4Projection proj4;

        public TestWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.Meter;
            //wpfMap1.CurrentExtent = new RectangleShape(-10000000, 5000000, 10000000, -5000000);
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

            proj4 = new ManagedProj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.FeatureSource.Projection = proj4;

            LayerOverlay layerOverlay = new LayerOverlay();
            layerOverlay.WrappingMode = WrappingMode.WrapDateline;

            worldLayer.Open();
            var wrappingExtent = worldLayer.GetBoundingBox();

            layerOverlay.WrappingExtent = wrappingExtent;
            layerOverlay.Layers.Add("worldLayer", worldLayer);
            wpfMap1.Overlays.Add("WorldOverlay", layerOverlay);

            InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            featureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
            featureLayer.WrappingMode = WrappingMode.WrapDateline;
            featureLayer.WrappingExtent = wrappingExtent;
            featureLayer.DrawingMarginPercentage = 30;
            //featureLayer.FeatureSource.Projection = proj4;

            FillFeatures(featureLayer);

            LayerOverlay overlay1 = new LayerOverlay();
            overlay1.WrappingMode = WrappingMode.WrapDateline;
            overlay1.WrappingExtent = wrappingExtent;
            overlay1.Layers.Add(featureLayer);
            wpfMap1.Overlays.Add(overlay1);

            wpfMap1.CurrentExtent = wrappingExtent;
            wpfMap1.ZoomLevelSet = GetCustomZoomLevelSet(591330330.94349861);
            wpfMap1.Refresh();
        }

        private void FillFeatures(InMemoryFeatureLayer featureLayer)
        {
            foreach (var point in GetTestPoints())
            {
                featureLayer.InternalFeatures.Add(new Feature(proj4.ConvertToExternalProjection(point)));
                //featureLayer.InternalFeatures.Add(new Feature(point));
            }
        }

        private static ZoomLevelSet GetCustomZoomLevelSet(double maxScale)
        {
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            int i = 0;
            while (i++ < 20)
            {
                zoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(maxScale));
                maxScale /= 2d;
            }

            return zoomLevelSet;
        }

        private IEnumerable<PointShape> GetTestPoints()
        {
            yield return new PointShape(180, -40);
            yield return new PointShape(179.8, 40);
            yield return new PointShape(179.3, 20);
            yield return new PointShape(50, 40);
        }

    }
}
